from aiohttp.web import Application
import pytest


async def test_fixture_test_server_get_properties(sanic_server):
    assert sanic_server.is_running is True
    assert sanic_server.has_started() is True
    assert sanic_server.port is not None
    url = sanic_server.make_url('/test')
    assert url == 'http://127.0.0.1:{port}/test'.format(port=str(sanic_server.port))


async def test_fixture_test_server_close(sanic_server):
    # close
    await sanic_server.close()
    assert sanic_server.is_running is False
    assert sanic_server.closed is True
    assert sanic_server.port is None


async def test_fixture_test_server_raise_exception_for_non_sanic_app(test_server):
    aiohttp_web = Application()
    with pytest.raises(TypeError):
        await test_server(aiohttp_web)


import pytest
from sanic.app import Sanic
from sanic.websocket import WebSocketProtocol
from sanic import response
from aiohttp.web import Application


async def test_fixture_test_client_get_properties(test_cli):
    assert test_cli.app is not None
    assert test_cli.host is not None
    assert test_cli.port is not None
    assert test_cli.server is not None
    assert test_cli.session is not None


async def test_fixture_test_client_make_url(test_cli):
    uri = '/test'
    url = test_cli.make_url(uri)
    assert url == "http://127.0.0.1:{port}/test".format(port=str(test_cli.port))


async def test_fixture_test_client_get(test_cli):
    resp = await test_cli.get('/test_get')
    assert resp.status == 200
    resp_json = await resp.json()
    assert resp_json == {"GET": True}


async def test_fixture_test_client_post(test_cli):
    resp = await test_cli.post('/test_post')
    assert resp.status == 200
    resp_json = await resp.json()
    assert resp_json == {"POST": True}


async def test_fixture_test_client_put(test_cli):
    resp = await test_cli.put('/test_put')
    assert resp.status == 200
    resp_json = await resp.json()
    assert resp_json == {"PUT": True}


async def test_fixture_test_client_delete(test_cli):
    resp = await test_cli.delete('/test_delete')
    assert resp.status == 200
    resp_json = await resp.json()
    assert resp_json == {"DELETE": True}


async def test_fixture_test_client_patch(test_cli):
    resp = await test_cli.patch('/test_patch')
    assert resp.status == 200
    resp_json = await resp.json()
    assert resp_json == {"PATCH": True}


async def test_fixture_test_client_options(test_cli):
    resp = await test_cli.options('/test_options')
    assert resp.status == 200
    resp_json = await resp.json()
    assert resp_json == {"OPTIONS": True}


async def test_fixture_test_client_head(test_cli):
    resp = await test_cli.head('/test_head')
    assert resp.status == 200
    resp_json = await resp.json()
    # HEAD should not have body
    assert resp_json is None


async def test_fixture_test_client_close(test_cli):
    resp = await test_cli.get('/test_get')
    assert resp.status == 200
    resp_json = await resp.json()
    assert resp_json == {"GET": True}
    await test_cli.close()
    assert test_cli._closed == True


async def test_fixture_test_client_passing_headers(test_cli):
    headers={"authorization": "Basic bG9naW46cGFzcw=="}
    resp = await test_cli.get('/test_passing_headers', headers=headers)
    assert resp.status == 200
    resp_json = await resp.json()
    assert resp_json["headers"]["authorization"] == headers["authorization"]


async def test_fixture_test_client_context_manager(app, test_client):
    async with await test_client(app) as test_cli:
        resp = await test_cli.get('/test_get')
        assert resp.status == 200
        resp_json = await resp.json()
        assert resp_json == {"GET": True}


async def test_fixture_test_client_raise_exception_for_non_sanic_app(test_client):
    aiohttp_web = Application()
    with pytest.raises(TypeError):
        await test_client(aiohttp_web)


async def test_fixture_test_client_app_is_running(test_cli):
    assert test_cli.app.is_running == True



import pytest
import asynctest
import asyncio
from unittest import mock
from async_generator import async_generator, yield_


@pytest.fixture
async def async_gen_fixture_sleep():
    await asyncio.sleep(0.1)
    yield_('a value')


@pytest.fixture
async def async_fixture_sleep():
    return await asyncio.sleep(0.1)


@pytest.fixture
async def async_fixture():
	client = mock.Mock()
	client.assume_role = asynctest.CoroutineMock(return_value={"foo": "bar"})
	return await client.assume_role()


async def test_async_fixture(async_fixture):
	assert async_fixture == {"foo": "bar"}


@pytest.fixture
def mock_fixture():
    return mock.Mock(return_value={"foo": "bar"})


@pytest.fixture
@async_generator
async def async_gen_fixture(mock_fixture):
    await yield_(mock_fixture())


async def test_async_gen_fixture(async_gen_fixture, mock_fixture):
    assert mock_fixture.called
    assert async_gen_fixture == {"foo": "bar"}



# This is a special test. it will fail for sanic <= 0.5.4
async def test_fixture_test_client_ws(test_cli):
    ws_conn = await test_cli.ws_connect('/test_ws')
    data = 'hello world!'
    await ws_conn.send_str(data)
    msg = await ws_conn.receive()
    assert msg.data == data
    await ws_conn.close()
